package com.fjhx.mes.service.production.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.item.entity.product.po.ProductInfo;
import com.fjhx.item.service.product.ProductInfoService;
import com.fjhx.mes.entity.bom.po.BomDetail;
import com.fjhx.mes.entity.bom.po.BomInfo;
import com.fjhx.mes.entity.production.dto.ProductionTaskDto;
import com.fjhx.mes.entity.production.dto.ProductionTaskSelectDto;
import com.fjhx.mes.entity.production.po.ProductionPlan;
import com.fjhx.mes.entity.production.po.ProductionTask;
import com.fjhx.mes.entity.production.vo.ProductionTaskVo;
import com.fjhx.mes.entity.work.po.WorkOrder;
import com.fjhx.mes.mapper.production.ProductionTaskMapper;
import com.fjhx.mes.service.bom.BomDetailService;
import com.fjhx.mes.service.bom.BomInfoService;
import com.fjhx.mes.service.production.ProductionPlanService;
import com.fjhx.mes.service.production.ProductionTaskService;
import com.fjhx.mes.service.work.WorkOrderService;
import com.fjhx.mes.utils.code.CodeEnum;
import com.fjhx.wms.entity.stock.emums.StockWaitType;
import com.fjhx.wms.entity.stock.po.StockWait;
import com.fjhx.wms.entity.stock.po.StockWaitDetails;
import com.fjhx.wms.service.stock.StockWaitDetailsService;
import com.fjhx.wms.service.stock.StockWaitService;
import com.obs.services.internal.ServiceException;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.ruoyi.system.utils.UserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


/**
 * <p>
 * 生产任务 服务实现类
 * </p>
 *
 * @author
 * @since 2023-03-30
 */
@Service
public class ProductionTaskServiceImpl extends ServiceImpl<ProductionTaskMapper, ProductionTask> implements ProductionTaskService {
    @Autowired
    private ProductInfoService productInfoService;
    @Autowired
    private ProductionPlanService productionPlanService;
    @Autowired
    private WorkOrderService workOrderService;
    @Autowired
    private BomInfoService bomInfoService;
    @Autowired
    private BomDetailService bomDetailService;
    @Autowired
    private StockWaitService stockWaitService;
    @Autowired
    private StockWaitDetailsService stockWaitDetailsService;

    @Override
    public Page<ProductionTaskVo> getPage(ProductionTaskSelectDto dto) {
        IWrapper<ProductionTask> wrapper = getWrapper();
        wrapper.orderByDesc("pt", ProductionTask::getId);
        if (ObjectUtil.isNotEmpty(dto.getKeyword())) {
            wrapper.and(q -> q.like("pt", ProductionTaskVo::getCode, dto.getKeyword())
                    .or().like("pt", ProductionTaskVo::getQuantity, dto.getKeyword())
                    .or().like("pt", ProductionTaskVo::getDueDate, dto.getKeyword()));
        }
        Page<ProductionTaskVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        List<ProductionTaskVo> records = page.getRecords();
        //赋值产品名称
        List<Long> productIds = records.stream().map(ProductionTaskVo::getProductId).collect(Collectors.toList());
        if (ObjectUtil.isNotEmpty(productIds)) {
            List<ProductInfo> productInfos = productInfoService.listByIds(productIds);
            Map<Long, ProductInfo> productInfoMap = productInfos.stream().collect(Collectors.groupingBy(ProductInfo::getId,
                    Collectors.collectingAndThen(Collectors.toList(), value -> value.get(0))));
            for (ProductionTaskVo productionTaskVo : records) {
                Long productId = productionTaskVo.getProductId();
                if (ObjectUtil.isNotEmpty(productId)) {
                    ProductInfo productInfo = productInfoMap.get(productId);
                    if (ObjectUtil.isNotEmpty(productInfo)) {
                        productionTaskVo.setProductName(productInfo.getName());
                    }
                }
            }
        }

        //赋值负责人名字
        UserUtil.assignmentNickName(records, ProductionTask::getPersonLiableId, ProductionTaskVo::setPersonLiableName);
        return page;
    }

    @Override
    public ProductionTaskVo detail(Long id) {
        ProductionTaskVo productionTaskVo = baseMapper.detail(id);
        ProductInfo productInfo = productInfoService.getById(productionTaskVo.getProductId());
        if (ObjectUtil.isNotEmpty(productInfo)) {
            productionTaskVo.setProductName(productInfo.getName());
        }
        Map<Long, String> nickNameMapByIds = UserUtil.getNickNameMapByIds(Arrays.asList(productionTaskVo.getPersonLiableId()));
        if (ObjectUtil.isNotEmpty(nickNameMapByIds)) {
            productionTaskVo.setPersonLiableName(nickNameMapByIds.get(productionTaskVo.getPersonLiableId()));
        }
        return productionTaskVo;
    }

    @DSTransactional
    @Override
    public void add(ProductionTaskDto productionTaskDto) {
        productionTaskDto.setCode(CodeEnum.PRODUCTION_TASK.getCode());
        this.save(productionTaskDto);
        Long productionPlanId = productionTaskDto.getProductionPlanId();
        //更新生产计划状态为进行中
        ProductionPlan productionPlan = productionPlanService.getById(productionPlanId);
        productionPlan.setStatus(1);
        productionPlanService.updateById(productionPlan);
        //添加待出库数据
        //查询工单中的产品信息
        WorkOrder workOrder = workOrderService.getById(productionPlan.getWorkOrderId());
        if (ObjectUtil.isEmpty(workOrder)) {
            throw new ServiceException("查询不到该任务的工单信息");
        }
        //查询BOM信息
        BomInfo bomInfo = bomInfoService.getOne(q -> q.eq(BomInfo::getProductId, workOrder.getProductId()));
        if (ObjectUtil.isEmpty(bomInfo)) {
            throw new ServiceException("查询不到产品的BOM信息");
        }
        List<BomDetail> bomDetailList = bomDetailService.list(q -> q.eq(BomDetail::getBomInfoId, bomInfo.getId()));
        if (ObjectUtil.isEmpty(bomDetailList)) {
            throw new ServiceException("查询不到BOM明细");
        }
        //创建待出库信息
        StockWait stockWait = new StockWait();
        stockWait.setBusinessId(productionTaskDto.getId());
        stockWait.setType(2);//出库
        stockWait.setStatus(0);//待出库
        stockWait.setBusinessType(StockWaitType.PRODUCTION_TASK_OUT.getDetailType());//生产任务出库
        stockWaitService.save(stockWait);
        //创建待出库明细
        List<StockWaitDetails> stockWaitDetailsList = new ArrayList<>();
        for (BomDetail bomDetail : bomDetailList) {
            //计算数量
            BigDecimal multiply = bomDetail.getQuantity().multiply(productionTaskDto.getQuantity());
            //创建待出库明细
            StockWaitDetails stockWaitDetails = new StockWaitDetails();
            stockWaitDetails.setStockWaitId(stockWait.getId());
            stockWaitDetails.setProductId(bomDetail.getProductId());
            stockWaitDetails.setQuantity(multiply);
            stockWaitDetails.setReceiptQuantity(BigDecimal.ZERO);
            stockWaitDetailsList.add(stockWaitDetails);
        }
        stockWaitDetailsService.saveBatch(stockWaitDetailsList);
    }

    @Override
    public void edit(ProductionTaskDto productionTaskDto) {
        this.updateById(productionTaskDto);
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

    @Override
    public List<ProductionTaskVo> myListByIds(List<Long> ids) {
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.in("pt.id", ids);
        return baseMapper.myListByIds(queryWrapper);
    }

}
